/*
 * SonarSource Language Recognizer
 * Copyright (C) 2010 SonarSource
 * dev@sonar.codehaus.org
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.sslr.internal.matchers;

/**
 * A {@link Matcher} that executes all of its submatchers in sequence.
 * Succeeds only if all submatchers succeed.
 */
public class SequenceMatcher implements Matcher {

  private final Matcher[] subMatchers;

  public SequenceMatcher(Matcher... subMatchers) {
    this.subMatchers = subMatchers;
  }

  public boolean match(MatcherContext context) {
    for (Matcher subMatcher : subMatchers) {
      if (!context.getSubContext(subMatcher).runMatcher()) {
        return false;
      }
    }
    context.skipNode();
    return true;
  }

  public Matcher[] getSubMatchers() {
    Matcher[] result = new Matcher[subMatchers.length];
    System.arraycopy(subMatchers, 0, result, 0, subMatchers.length);
    return result;
  }

}
